@@ -5,7 +5,7 @@ from zope.interface import (
     Interface,
     )
 
-from pyramid.compat import PY3
+from pyramid.compat import PY2
 
 # public API interfaces
 
@@ -25,6 +25,14 @@ class IContextFound(Interface):
 
 IAfterTraversal = IContextFound
 
+class IBeforeTraversal(Interface):
+    """
+    An event type that is emitted after :app:`Pyramid` attempted to find a
+    route but before it calls any traversal or view code. See the documentation
+    attached to :class:`pyramid.events.Routefound` for more information.
+    """
+    request = Attribute('The request object')
+
 class INewRequest(Interface):
     """ An event type that is emitted whenever :app:`Pyramid`
     begins to process a new request.  See the documentation attached
@@ -149,7 +157,7 @@ class IResponse(Interface):
         """Gets and sets and deletes the Content-MD5 header. For more
         information on Content-MD5 see RFC 2616 section 14.14.""")
 
-    content_range  = Attribute(
+    content_range = Attribute(
         """Gets and sets and deletes the Content-Range header. For more
         information on Content-Range see section 14.16. Converts using
         ContentRange object.""")
@@ -172,7 +180,7 @@ class IResponse(Interface):
         """Gets and sets and deletes the Date header. For more information on
         Date see RFC 2616 section 14.18. Converts using HTTP date.""")
 
-    def delete_cookie(key, path='/', domain=None):
+    def delete_cookie(name, path='/', domain=None):
         """ Delete a cookie from the client. Note that path and domain must
         match how the cookie was originally set.  This sets the cookie to the
         empty string, and max_age=0 so that it should expire immediately. """
@@ -236,7 +244,7 @@ class IResponse(Interface):
         """ Gets and sets and deletes the Server header. For more information
         on Server see RFC216 section 14.38. """)
 
-    def set_cookie(key, value='', max_age=None, path='/', domain=None,
+    def set_cookie(name, value='', max_age=None, path='/', domain=None,
                    secure=False, httponly=False, comment=None, expires=None,
                    overwrite=False):
         """ Set (add) a cookie for the response """
@@ -251,7 +259,7 @@ class IResponse(Interface):
         """ Get/set the unicode value of the body (using the charset of
         the Content-Type)""")
 
-    def unset_cookie(key, strict=True):
+    def unset_cookie(name, strict=True):
         """ Unset a cookie with the given name (remove it from the
         response)."""
 
@@ -311,7 +319,7 @@ class IDict(Interface):
     def values():
         """ Return a list of values from the dictionary """
 
-    if not PY3:
+    if PY2:
 
         def iterkeys():
             """ Return an iterator of keys from the dictionary """
@@ -382,6 +390,9 @@ class IRendererInfo(Interface):
     settings = Attribute('The deployment settings dictionary related '
                          'to the current application')
 
+    def clone():
+        """ Return a shallow copy that does not share any mutable state."""
+
 class IRendererFactory(Interface):
     def __call__(info):
         """ Return an object that implements
@@ -439,42 +450,63 @@ class IViewMapperFactory(Interface):
 
 class IAuthenticationPolicy(Interface):
     """ An object representing a Pyramid authentication policy. """
+
     def authenticated_userid(request):
-        """ Return the authenticated userid or ``None`` if no authenticated
-        userid can be found. This method of the policy should ensure that a
-        record exists in whatever persistent store is used related to the
-        user (the user should not have been deleted); if a record associated
-        with the current id does not exist in a persistent store, it should
-        return ``None``."""
+        """ Return the authenticated :term:`userid` or ``None`` if
+        no authenticated userid can be found. This method of the
+        policy should ensure that a record exists in whatever
+        persistent store is used related to the user (the user
+        should not have been deleted); if a record associated with
+        the current id does not exist in a persistent store, it
+        should return ``None``.
+
+        """
 
     def unauthenticated_userid(request):
-        """ Return the *unauthenticated* userid.  This method performs the
-        same duty as ``authenticated_userid`` but is permitted to return the
-        userid based only on data present in the request; it needn't (and
-        shouldn't) check any persistent store to ensure that the user record
-        related to the request userid exists."""
+        """ Return the *unauthenticated* userid.  This method
+        performs the same duty as ``authenticated_userid`` but is
+        permitted to return the userid based only on data present
+        in the request; it needn't (and shouldn't) check any
+        persistent store to ensure that the user record related to
+        the request userid exists.
+
+        This method is intended primarily a helper to assist the
+        ``authenticated_userid`` method in pulling credentials out
+        of the request data, abstracting away the specific headers,
+        query strings, etc that are used to authenticate the request.
+
+        """
 
     def effective_principals(request):
         """ Return a sequence representing the effective principals
-        including the userid and any groups belonged to by the current
-        user, including 'system' groups such as Everyone and
-        Authenticated. """
+        typically including the :term:`userid` and any groups belonged
+        to by the current user, always including 'system' groups such
+        as ``pyramid.security.Everyone`` and
+        ``pyramid.security.Authenticated``.
 
-    def remember(request, principal, **kw):
+        """
+
+    def remember(request, userid, **kw):
         """ Return a set of headers suitable for 'remembering' the
-        principal named ``principal`` when set in a response.  An
-        individual authentication policy and its consumers can decide
-        on the composition and meaning of ``**kw.`` """
+        :term:`userid` named ``userid`` when set in a response.  An
+        individual authentication policy and its consumers can
+        decide on the composition and meaning of ``**kw``.
+
+        """
 
     def forget(request):
         """ Return a set of headers suitable for 'forgetting' the
-        current user on subsequent requests. """
+        current user on subsequent requests.
+
+        """
 
 class IAuthorizationPolicy(Interface):
     """ An object representing a Pyramid authorization policy. """
     def permits(context, principals, permission):
-        """ Return ``True`` if any of the ``principals`` is allowed the
-        ``permission`` in the current ``context``, else return ``False``
+        """ Return an instance of :class:`pyramid.security.Allowed` if any
+        of the ``principals`` is allowed the ``permission`` in the current
+        ``context``, else return an instance of
+        :class:`pyramid.security.Denied`.
         """
 
     def principals_allowed_by_permission(context, permission):
@@ -562,19 +594,20 @@ class IStaticURLInfo(Interface):
     def generate(path, request, **kw):
         """ Generate a URL for the given path """
 
+    def add_cache_buster(config, spec, cache_buster):
+        """ Add a new cache buster to a particular set of assets """
+
 class IResponseFactory(Interface):
-    """ A utility which generates a response factory """
-    def __call__():
-        """ Return a response factory (e.g. a callable that returns an object
-        implementing IResponse, e.g. :class:`pyramid.response.Response`). It
-        should accept all the arguments that the Pyramid Response class
-        accepts."""
+    """ A utility which generates a response """
+    def __call__(request):
+        """ Return a response object implementing IResponse,
+        e.g. :class:`pyramid.response.Response`). It should handle the
+        case when ``request`` is ``None``."""
 
 class IRequestFactory(Interface):
     """ A utility which generates a request """
     def __call__(environ):
-        """ Return an object implementing IRequest, e.g. an instance
-        of ``pyramid.request.Request``"""
+        """ Return an instance of ``pyramid.request.Request``"""
 
     def blank(path):
         """ Return an empty request object (see
@@ -646,12 +679,83 @@ class IViewPermission(Interface):
         """
 
 class IRouter(Interface):
-    """ WSGI application which routes requests to 'view' code based on
-    a view registry."""
+    """
+    WSGI application which routes requests to 'view' code based on
+    a view registry.
+
+    """
     registry = Attribute(
         """Component architecture registry local to this application.""")
 
-class ISettings(Interface):
+    def request_context(environ):
+        """
+        Create a new request context from a WSGI environ.
+
+        The request context is used to push/pop the threadlocals required
+        when processing the request. It also contains an initialized
+        :class:`pyramid.interfaces.IRequest` instance using the registered
+        :class:`pyramid.interfaces.IRequestFactory`. The context may be
+        used as a context manager to control the threadlocal lifecycle:
+
+        .. code-block:: python
+
+            with router.request_context(environ) as request:
+                ...
+
+        Alternatively, the context may be used without the ``with`` statement
+        by manually invoking its ``begin()`` and ``end()`` methods.
+
+        .. code-block:: python
+
+            ctx = router.request_context(environ)
+            request = ctx.begin()
+            try:
+                ...
+            finally:
+                ctx.end()
+
+        """
+
+    def invoke_request(request):
+        """
+        Invoke the :app:`Pyramid` request pipeline.
+
+        See :ref:`router_chapter` for information on the request pipeline.
+
+        The output should be a :class:`pyramid.interfaces.IResponse` object
+        or a raised exception.
+
+        """
+
+class IExecutionPolicy(Interface):
+    def __call__(environ, router):
+        """
+        This callable triggers the router to process a raw WSGI environ dict
+        into a response and controls the :app:`Pyramid` request pipeline.
+
+        The ``environ`` is the raw WSGI environ.
+
+        The ``router`` is an :class:`pyramid.interfaces.IRouter` object which
+        should be used to create a request object and send it into the
+        processing pipeline.
+
+        The return value should be a :class:`pyramid.interfaces.IResponse`
+        object or an exception that will be handled by WSGI middleware.
+
+        The default execution policy simply creates a request and sends it
+        through the pipeline, attempting to render any exception that escapes:
+
+        .. code-block:: python
+
+            def simple_execution_policy(environ, router):
+                with router.request_context(environ) as request:
+                    try:
+                        return router.invoke_request(request)
+                    except Exception:
+                        return request.invoke_exception_view(reraise=True)
+        """
+
+class ISettings(IDict):
     """ Runtime settings utility for pyramid; represents the
     deployment settings for the application.  Implements a mapping
     interface."""
@@ -708,7 +812,7 @@ class IRoute(Interface):
     pregenerator = Attribute('This attribute should either be ``None`` or '
                              'a callable object implementing the '
                              '``IRoutePregenerator`` interface')
-        
+
     def match(path):
         """
         If the ``path`` passed to this function can be matched by the
@@ -768,58 +872,6 @@ class IResourceURL(Interface):
         'The physical url path of the resource as a tuple. (New in 1.5)'
         )
 
-class IContextURL(IResourceURL):
-    """
-    .. deprecated:: 1.3
-        An adapter which deals with URLs related to a context.  Use
-        :class:`pyramid.interfaces.IResourceURL` instead.
-    """
-    # this class subclasses IResourceURL because request.resource_url looks
-    # for IResourceURL via queryAdapter.  queryAdapter will find a deprecated
-    # IContextURL registration if no registration for IResourceURL exists.
-    # In reality, however, IContextURL objects were never required to have
-    # the virtual_path or physical_path attributes spelled in IResourceURL.
-    # The inheritance relationship is purely to benefit adapter lookup,
-    # not to imply an inheritance relationship of interface attributes
-    # and methods.
-    #
-    # Mechanics:
-    #
-    # class Fudge(object):
-    #     def __init__(self, one, two):
-    #         print(one, two)
-    # class Another(object):
-    #     def __init__(self, one, two):
-    #         print(one, two)
-    # ob = object()
-    # r.registerAdapter(Fudge, (Interface, Interface), IContextURL)
-    # print(r.queryMultiAdapter((ob, ob), IResourceURL))
-    # r.registerAdapter(Another, (Interface, Interface), IResourceURL)
-    # print(r.queryMultiAdapter((ob, ob), IResourceURL))
-    #
-    # prints
-    #
-    # <object object at 0x7fa678f3e2a0> <object object at 0x7fa678f3e2a0>
-    # <__main__.Fudge object at 0x1cda890>
-    # <object object at 0x7fa678f3e2a0> <object object at 0x7fa678f3e2a0>
-    # <__main__.Another object at 0x1cda850>
-    
-    def virtual_root():
-        """ Return the virtual root related to a request and the
-        current context"""
-
-    def __call__():
-        """ Return a URL that points to the context. """
-
-deprecated(
-    'IContextURL',
-    'As of Pyramid 1.3 the, "pyramid.interfaces.IContextURL" interface is '
-    'scheduled to be removed.   Use the '
-    '"pyramid.config.Configurator.add_resource_url_adapter" method to register '
-    'a class that implements "pyramid.interfaces.IResourceURL" instead. '
-    'See the "What\'s new In Pyramid 1.3" document for more details.'
-    )
-
 class IPEP302Loader(Interface):
     """ See http://www.python.org/dev/peps/pep-0302/#id30.
     """
@@ -837,9 +889,9 @@ class IPEP302Loader(Interface):
 
     def get_code(fullname):
         """ Return the code object for the module identified by 'fullname'.
-        
+
         Return 'None' if it's a built-in or extension module.
-        
+
         If the loader doesn't have the code object but it does have the source
         code, return the compiled source code.
 
@@ -848,16 +900,16 @@ class IPEP302Loader(Interface):
 
     def get_source(fullname):
         """ Return the source code for the module identified by 'fullname'.
-        
+
         Return a string, using newline characters for line endings, or None
         if the source is not available.
-            
+
         Raise ImportError if the module can't be found by the importer at all.
         """
 
     def get_filename(fullname):
         """ Return the value of '__file__' if the named module was loaded.
-        
+
         If the module is not found, raise ImportError.
         """
 
@@ -885,6 +937,17 @@ class IDefaultPermission(Interface):
     for all view configurations which do not explicitly declare their
     own."""
 
+class IDefaultCSRFOptions(Interface):
+    """ An object representing the default CSRF settings to be used for
+    all view configurations which do not explicitly declare their own."""
+    require_csrf = Attribute(
+        'Boolean attribute. If ``True``, then CSRF checks will be enabled by '
+        'default for the view unless overridden.')
+    token = Attribute('The key to be matched in the body of the request.')
+    header = Attribute('The header to be matched with the CSRF token.')
+    safe_methods = Attribute('A set of safe methods that skip CSRF checks.')
+    callback = Attribute('A callback to disable CSRF checks per-request.')
+
 class ISessionFactory(Interface):
     """ An interface representing a factory which accepts a request object and
     returns an ISession object """
@@ -896,6 +959,13 @@ class ISession(IDict):
     usually accessed via ``request.session``.
 
     Keys and values of a session must be pickleable.
+
+    .. versionchanged:: 1.9
+
+       Sessions are no longer required to implement ``get_csrf_token`` and
+       ``new_csrf_token``. CSRF token support was moved to the pluggable
+       :class:`pyramid.interfaces.ICSRFStoragePolicy` configuration hook.
+
     """
 
     # attributes
@@ -950,19 +1020,39 @@ class ISession(IDict):
         :meth:`pyramid.interfaces.ISession.flash`
         """
 
-    def new_csrf_token():
-        """ Create and set into the session a new, random cross-site request
-        forgery protection token.  Return the token.  It will be a string."""
 
-    def get_csrf_token():
-        """ Return a random cross-site request forgery protection token.  It
-        will be a string.  If a token was previously added to the session via
-        ``new_csrf_token``, that token will be returned.  If no CSRF token
-        was previously set into the session, ``new_csrf_token`` will be
+class ICSRFStoragePolicy(Interface):
+    """ An object that offers the ability to verify CSRF tokens and generate
+    new ones."""
+
+    def new_csrf_token(request):
+        """ Create and return a new, random cross-site request forgery
+        protection token. The token will be an ascii-compatible unicode
+        string.
+
+        """
+
+    def get_csrf_token(request):
+        """ Return a cross-site request forgery protection token.  It
+        will be an ascii-compatible unicode string.  If a token was previously
+        set for this user via ``new_csrf_token``, that token will be returned.
+        If no CSRF token was previously set, ``new_csrf_token`` will be
         called, which will create and set a token, and this token will be
         returned.
+
         """
 
+    def check_csrf_token(request, token):
+        """ Determine if the supplied ``token`` is valid. Most implementations
+        should simply compare the ``token`` to the current value of
+        ``get_csrf_token`` but it is possible to verify the token using
+        any mechanism necessary using this method.
+
+        Returns ``True`` if the ``token`` is valid, otherwise ``False``.
+
+        """
+
+
 class IIntrospector(Interface):
     def get(category_name, discriminator, default=None):
         """ Get the IIntrospectable related to the category_name and the
@@ -1164,10 +1254,86 @@ class IJSONAdapter(Interface):
 class IPredicateList(Interface):
     """ Interface representing a predicate list """
 
+class IViewDeriver(Interface):
+    options = Attribute('A list of supported options to be passed to '
+                        ':meth:`pyramid.config.Configurator.add_view`. '
+                        'This attribute is optional.')
+
+    def __call__(view, info):
+        """
+        Derive a new view from the supplied view.
+
+        View options, package information and registry are available on
+        ``info``, an instance of :class:`pyramid.interfaces.IViewDeriverInfo`.
+
+        The ``view`` is a callable accepting ``(context, request)``.
+
+        """
+
+class IViewDeriverInfo(Interface):
+    """ An object implementing this interface is passed to every
+    :term:`view deriver` during configuration."""
+    registry = Attribute('The "current" application registry where the '
+                         'view was created')
+    package = Attribute('The "current package" where the view '
+                        'configuration statement was found')
+    settings = Attribute('The deployment settings dictionary related '
+                         'to the current application')
+    options = Attribute('The view options passed to the view, including any '
+                        'default values that were not overriden')
+    predicates = Attribute('The list of predicates active on the view')
+    original_view = Attribute('The original view object being wrapped')
+    exception_only = Attribute('The view will only be invoked for exceptions')
+
+class IViewDerivers(Interface):
+    """ Interface for view derivers list """
+
+class ICacheBuster(Interface):
+    """
+    A cache buster modifies the URL generation machinery for
+    :meth:`~pyramid.request.Request.static_url`. See :ref:`cache_busting`.
+
+    .. versionadded:: 1.6
+    """
+    def __call__(request, subpath, kw):
+        """
+        Modifies a subpath and/or keyword arguments from which a static asset
+        URL will be computed during URL generation.
+
+        The ``subpath`` argument is a path of ``/``-delimited segments that
+        represent the portion of the asset URL which is used to find the asset.
+        The ``kw`` argument is a dict of keywords that are to be passed
+        eventually to :meth:`~pyramid.request.Request.static_url` for URL
+        generation.  The return value should be a two-tuple of
+        ``(subpath, kw)`` where ``subpath`` is the relative URL from where the
+        file is served and ``kw`` is the same input argument. The return value
+        should be modified to include the cache bust token in the generated
+        URL.
+
+        The ``kw`` dictionary contains extra arguments passed to
+        :meth:`~pyramid.request.Request.static_url` as well as some extra
+        items that may be usful including:
+
+          - ``pathspec`` is the path specification for the resource
+            to be cache busted.
+
+          - ``rawspec`` is the original location of the file, ignoring
+            any calls to :meth:`pyramid.config.Configurator.override_asset`.
+
+        The ``pathspec`` and ``rawspec`` values are only different in cases
+        where an asset has been mounted into a virtual location using
+        :meth:`pyramid.config.Configurator.override_asset`. For example, with
+        a call to ``request.static_url('myapp:static/foo.png'), the
+        ``pathspec`` is ``myapp:static/foo.png`` whereas the ``rawspec`` may
+        be ``themepkg:bar.png``, assuming a call to
+        ``config.override_asset('myapp:static/foo.png', 'themepkg:bar.png')``.
+        """
+
 # configuration phases: a lower phase number means the actions associated
 # with this phase will be executed earlier than those with later phase
 # numbers.  The default phase number is 0, FTR.
 
+PHASE0_CONFIG = -30
 PHASE1_CONFIG = -20
 PHASE2_CONFIG = -10
-
+PHASE3_CONFIG = 0
