@@ -12,16 +12,20 @@ from webob.cookies import SignedSerializer
 
 from pyramid.compat import (
     pickle,
-    PY3,
+    PY2,
     text_,
     bytes_,
     native_,
     )
+from pyramid.csrf import (
+    check_csrf_origin,
+    check_csrf_token,
+)
 
-from pyramid.exceptions import BadCSRFToken
 from pyramid.interfaces import ISession
 from pyramid.util import strings_differ
 
+
 def manage_accessed(wrapped):
     """ Decorator which causes a cookie to be renewed when an accessor
     method is called."""
@@ -101,45 +105,27 @@ def signed_deserialize(serialized, secret, hmac=hmac):
 
     return pickle.loads(pickled)
 
-def check_csrf_token(request,
-                     token='csrf_token',
-                     header='X-CSRF-Token',
-                     raises=True):
-    """ Check the CSRF token in the request's session against the value in
-    ``request.params.get(token)`` or ``request.headers.get(header)``.
-    If a ``token`` keyword is not supplied to this function, the string
-    ``csrf_token`` will be used to look up the token in ``request.params``.
-    If a ``header`` keyword is not supplied to this function, the string
-    ``X-CSRF-Token`` will be used to look up the token in ``request.headers``.
-
-    If the value supplied by param or by header doesn't match the value
-    supplied by ``request.session.get_csrf_token()``, and ``raises`` is
-    ``True``, this function will raise an
-    :exc:`pyramid.exceptions.BadCSRFToken` exception.
-    If the check does succeed and ``raises`` is ``False``, this
-    function will return ``False``.  If the CSRF check is successful, this
-    function will return ``True`` unconditionally.
-
-    Note that using this function requires that a :term:`session factory` is
-    configured.
-
-    .. versionadded:: 1.4a2
-    """
-    supplied_token = request.params.get(token, request.headers.get(header, ""))
-    if strings_differ(request.session.get_csrf_token(), supplied_token):
-        if raises:
-            raise BadCSRFToken('check_csrf_token(): Invalid token')
-        return False
-    return True
 
 class PickleSerializer(object):
-    """ A Webob cookie serializer that uses the pickle protocol to dump Python
-    data to bytes."""
+    """ A serializer that uses the pickle protocol to dump Python
+    data to bytes.
+
+    This is the default serializer used by Pyramid.
+
+    ``protocol`` may be specified to control the version of pickle used.
+    Defaults to :attr:`pickle.HIGHEST_PROTOCOL`.
+
+    """
+    def __init__(self, protocol=pickle.HIGHEST_PROTOCOL):
+        self.protocol = protocol
+
     def loads(self, bstruct):
+        """Accept bytes and return a Python object."""
         return pickle.loads(bstruct)
 
     def dumps(self, appstruct):
-        return pickle.dumps(appstruct, pickle.HIGHEST_PROTOCOL)
+        """Accept a Python object and return bytes."""
+        return pickle.dumps(appstruct, self.protocol)
 
 def BaseCookieSessionFactory(
     serializer,
@@ -155,7 +141,7 @@ def BaseCookieSessionFactory(
     ):
     """
     .. versionadded:: 1.5
-    
+
     Configure a :term:`session factory` which will provide cookie-based
     sessions.  The return value of this function is a :term:`session factory`,
     which may be provided as the ``session_factory`` argument of a
@@ -238,14 +224,14 @@ def BaseCookieSessionFactory(
 
         # configuration parameters
         _cookie_name = cookie_name
-        _cookie_max_age = max_age
+        _cookie_max_age = max_age if max_age is None else int(max_age)
         _cookie_path = path
         _cookie_domain = domain
         _cookie_secure = secure
         _cookie_httponly = httponly
         _cookie_on_exception = set_on_exception
-        _timeout = timeout
-        _reissue_time = reissue_time
+        _timeout = timeout if timeout is None else int(timeout)
+        _reissue_time = reissue_time if reissue_time is None else int(reissue_time)
 
         # dirty flag
         _dirty = False
@@ -313,7 +299,7 @@ def BaseCookieSessionFactory(
         __len__ = manage_accessed(dict.__len__)
         __iter__ = manage_accessed(dict.__iter__)
 
-        if not PY3:
+        if PY2:
             iteritems = manage_accessed(dict.iteritems)
             itervalues = manage_accessed(dict.itervalues)
             iterkeys = manage_accessed(dict.iterkeys)
@@ -407,7 +393,7 @@ def UnencryptedCookieSessionFactoryConfig(
         compatible with cookies generated using
         ``UnencryptedCookieSessionFactory``, so existing user session data
         will be destroyed if you switch to it.
-    
+
     Configure a :term:`session factory` which will provide unencrypted
     (but signed) cookie-based sessions.  The return value of this
     function is a :term:`session factory`, which may be provided as
@@ -466,7 +452,7 @@ def UnencryptedCookieSessionFactoryConfig(
     class SerializerWrapper(object):
         def __init__(self, secret):
             self.secret = secret
-            
+
         def loads(self, bstruct):
             return signed_deserialize(bstruct, secret)
 
@@ -514,7 +500,7 @@ def SignedCookieSessionFactory(
     ):
     """
     .. versionadded:: 1.5
-    
+
     Configure a :term:`session factory` which will provide signed
     cookie-based sessions.  The return value of this
     function is a :term:`session factory`, which may be provided as
@@ -533,7 +519,7 @@ def SignedCookieSessionFactory(
     ``secret``
       A string which is used to sign the cookie. The secret should be at
       least as long as the block size of the selected hash algorithm. For
-      ``sha512`` this would mean a 128 bit (64 character) secret.  It should
+      ``sha512`` this would mean a 512 bit (64 character) secret.  It should
       be unique within the set of secret values provided to Pyramid for
       its various subsystems (see :ref:`admonishment_against_secret_sharing`).
 
@@ -626,3 +612,13 @@ def SignedCookieSessionFactory(
         reissue_time=reissue_time,
         set_on_exception=set_on_exception,
     )
+
+check_csrf_origin = check_csrf_origin  # api
+deprecated('check_csrf_origin',
+           'pyramid.session.check_csrf_origin is deprecated as of Pyramid '
+           '1.9. Use pyramid.csrf.check_csrf_origin instead.')
+
+check_csrf_token = check_csrf_token  # api
+deprecated('check_csrf_token',
+           'pyramid.session.check_csrf_token is deprecated as of Pyramid '
+           '1.9. Use pyramid.csrf.check_csrf_token instead.')
